#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

//*********************************************************************
// Programmed by Finn Bjurvoll Hansen - www.jurbol.com
// Email: jurboltrading@gmail.com
//*********************************************************************


namespace NinjaTrader.Indicator
{
    
    [Description("Displaced SMA indicators with colorcoded close above/below the DSMA. Selectable priceline also included.")]
    public class _Jurbol_TDI_DSMA : Indicator
    {
        #region Variables
      	private int 		period 			 = 5; 
        private int 		displace	 	 = 2; 
		private bool		colors		     = true;
		private bool		priceLine		 = true;
		private Color		up	   	         = Color.DeepSkyBlue;
		private Color		dn		   	     = Color.Red;
		private Color	  	lineColorBull 	 = Color.Lime;
		private Color	  	lineColorBear 	 = Color.Red;
		private Color	  	lineColorFlat  	 = Color.Black;
		private int		  	lineWidth 		 = 2;
		private DashStyle lineStyle   		 = DashStyle.Dot; 
		
		private int rayLength				 = 5;
       
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.Yellow), PlotStyle.Line, "DSMA"));
			
			Plots[0].Pen.Width		= 2; // DSMA
			Plots[0].Pen.DashStyle	= DashStyle.Dash;
            Overlay				= true;
			CalculateOnBarClose	= false;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
         {if 
		(CurrentBar < 5) 
				return;
            
		DSMA.Set(SMA(Typical,period)[displace]);
		
		if(Close[0] > DSMA[0]){PlotColors[0][0] = up;}
		if(Close[0] < DSMA[0]){PlotColors[0][0] = dn;}
		
		if (rayLength > 20)
			rayLength = 20;
		
		if (CurrentBar > 50)
		if(priceLine)
		if(Close[0] > Open[0])
		{
		DrawRay("CurrRay", false, rayLength, Close[0], 0, Close[0], lineColorBull, lineStyle, lineWidth);
		}
		else if(Close[0] < Open[0])
		{
		DrawRay("CurrRay", false, rayLength, Close[0], 0, Close[0], lineColorBear, lineStyle, lineWidth);
		}
		else
		{
		DrawRay("CurrRay", false, rayLength, Close[0], 0, Close[0], lineColorFlat, lineStyle, lineWidth);
		}
			
        }
		public override string ToString()
		{
			Name= "TDI DSMA © Jurbol.com";
			/* We return a label consisting of only the parameters we want to include. If we did not
			override this method we would have a label with all four parameters. In this example we will
			be left with only the parameters "Period" and "Interval". */
			return Name;
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries DSMA
        {
            get { return Values[0]; }
        }
		
		[Description("SMA Period")]
		[GridCategory("01. DSMA Settings")]
		[Gui.Design.DisplayNameAttribute("01. SMA Period)")]
		public int Period
		{
			get{return period;}
			set{period = Math.Max(0,value);}
		}

        [Description("Displacement")]
		[GridCategory("01. DSMA Settings")]
		[Gui.Design.DisplayNameAttribute("02. Displacement (Max 5)")]
		public int Displace
		{
			get{return displace;}
			set{displace = Math.Max(0,value);}
		}
		
		[Description("Colorcode the DSMA?")]
		[GridCategory("01. DSMA Settings")]
		[Gui.Design.DisplayNameAttribute("03. Colorcode the DSMA?")]
		public bool Colors
		{
			get { return colors; }
			set { colors = value; }
		}
		
		[XmlIgnore()]
        [Description("Color for close above DSMA")]
        [GridCategory("01. DSMA Settings")]
		[Gui.Design.DisplayNameAttribute("04. Close Above DSMA")]
        public Color Up
        {
            get { return up; }
            set { up = value; }
        }
		[Browsable(false)]
		public string UpSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(Up); }
			set { Up = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		[XmlIgnore()]
        [Description("Color for close below DSMA")]
        [GridCategory("01. DSMA Settings")]
		[Gui.Design.DisplayNameAttribute("05. Close Below DSMA")]
        public Color Dn
        {
            get { return dn; }
            set { dn = value; }
        }
		[Browsable(false)]
		public string DnSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(Dn); }
			set { Dn = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		[Description("Show or Hide the priceline")]
        [GridCategory("02. Priceline Settings")]
		[Gui.Design.DisplayName ("01. Display Priceline?")]
        public bool PriceLine
        {
            get { return priceLine; }
            set { priceLine = value; }
        }
		
		[Description("LineStyle for Price Line")]
        [GridCategory("02. Priceline Settings")]
		[Gui.Design.DisplayName ("02. Price Line Style")]
        public DashStyle LineStyle
        {
            get { return lineStyle; }
            set { lineStyle = value; }
        }
		
		[Description("Width of the Price Line")]
    	[Category("Plots")]
		[Gui.Design.DisplayName ("03. Price Line Width")]
        public int LineWidth
        {
            get { return lineWidth; }
			set { lineWidth = Math.Max(1, value); }
        }
		
     	[Description("Line length....(for Ray only)....# of bars from right edge  NOTE:  Only works if Use Line = false")]
    	[GridCategory("02. Priceline Settings")]
		[Gui.Design.DisplayName ("04. Line Length for Ray")]
        public int RayLength
        {
            get { return rayLength; }
			set { rayLength = Math.Max(1, value); }
        }
		
		[XmlIgnore()]
        [Description("Color of the Price Line")]
        [GridCategory("02. Priceline Settings")]
		[Gui.Design.DisplayName ("05. Bull Line Color")]
        public Color LineColorBull
        {
            get { return lineColorBull; }
            set { lineColorBull = value; }
        }
			[Browsable(false)]
			public string LineColorBullSerialize
		{
		get { return NinjaTrader.Gui.Design.SerializableColor.ToString(lineColorBull); }//SerializableColor
		set { lineColorBull = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}	
		
		[XmlIgnore()]
        [Description("Color of the Price Line")]
        [GridCategory("02. Priceline Settings")]
		[Gui.Design.DisplayName ("06. Bear Line Color")]
        public Color LineColorBear
        {
            get { return lineColorBear; }
            set { lineColorBear = value; }
        }
			[Browsable(false)]
			public string LineColorBearSerialize
		{
		get { return NinjaTrader.Gui.Design.SerializableColor.ToString(lineColorBear); }//SerializableColor
		set { lineColorBear = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		[XmlIgnore()]
        [Description("Color of the Price Line")]
        [GridCategory("02. Priceline Settings")]
		[Gui.Design.DisplayName ("07. Flat Line Color")]
        public Color LineColorFlat
        {
            get { return lineColorFlat; }
            set { lineColorFlat = value; }
        }
			[Browsable(false)]
			public string LineColorFlatSerialize
		{
		get { return NinjaTrader.Gui.Design.SerializableColor.ToString(lineColorFlat); }//SerializableColor
		set { lineColorFlat = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}	
		
		
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private _Jurbol_TDI_DSMA[] cache_Jurbol_TDI_DSMA = null;

        private static _Jurbol_TDI_DSMA check_Jurbol_TDI_DSMA = new _Jurbol_TDI_DSMA();

        /// <summary>
        /// Displaced SMA indicators with colorcoded close above/below the DSMA. Selectable priceline also included.
        /// </summary>
        /// <returns></returns>
        public _Jurbol_TDI_DSMA _Jurbol_TDI_DSMA(bool colors, int displace, Color dn, Color lineColorBear, Color lineColorBull, Color lineColorFlat, DashStyle lineStyle, int period, bool priceLine, int rayLength, Color up)
        {
            return _Jurbol_TDI_DSMA(Input, colors, displace, dn, lineColorBear, lineColorBull, lineColorFlat, lineStyle, period, priceLine, rayLength, up);
        }

        /// <summary>
        /// Displaced SMA indicators with colorcoded close above/below the DSMA. Selectable priceline also included.
        /// </summary>
        /// <returns></returns>
        public _Jurbol_TDI_DSMA _Jurbol_TDI_DSMA(Data.IDataSeries input, bool colors, int displace, Color dn, Color lineColorBear, Color lineColorBull, Color lineColorFlat, DashStyle lineStyle, int period, bool priceLine, int rayLength, Color up)
        {
            if (cache_Jurbol_TDI_DSMA != null)
                for (int idx = 0; idx < cache_Jurbol_TDI_DSMA.Length; idx++)
                    if (cache_Jurbol_TDI_DSMA[idx].Colors == colors && cache_Jurbol_TDI_DSMA[idx].Displace == displace && cache_Jurbol_TDI_DSMA[idx].Dn == dn && cache_Jurbol_TDI_DSMA[idx].LineColorBear == lineColorBear && cache_Jurbol_TDI_DSMA[idx].LineColorBull == lineColorBull && cache_Jurbol_TDI_DSMA[idx].LineColorFlat == lineColorFlat && cache_Jurbol_TDI_DSMA[idx].LineStyle == lineStyle && cache_Jurbol_TDI_DSMA[idx].Period == period && cache_Jurbol_TDI_DSMA[idx].PriceLine == priceLine && cache_Jurbol_TDI_DSMA[idx].RayLength == rayLength && cache_Jurbol_TDI_DSMA[idx].Up == up && cache_Jurbol_TDI_DSMA[idx].EqualsInput(input))
                        return cache_Jurbol_TDI_DSMA[idx];

            lock (check_Jurbol_TDI_DSMA)
            {
                check_Jurbol_TDI_DSMA.Colors = colors;
                colors = check_Jurbol_TDI_DSMA.Colors;
                check_Jurbol_TDI_DSMA.Displace = displace;
                displace = check_Jurbol_TDI_DSMA.Displace;
                check_Jurbol_TDI_DSMA.Dn = dn;
                dn = check_Jurbol_TDI_DSMA.Dn;
                check_Jurbol_TDI_DSMA.LineColorBear = lineColorBear;
                lineColorBear = check_Jurbol_TDI_DSMA.LineColorBear;
                check_Jurbol_TDI_DSMA.LineColorBull = lineColorBull;
                lineColorBull = check_Jurbol_TDI_DSMA.LineColorBull;
                check_Jurbol_TDI_DSMA.LineColorFlat = lineColorFlat;
                lineColorFlat = check_Jurbol_TDI_DSMA.LineColorFlat;
                check_Jurbol_TDI_DSMA.LineStyle = lineStyle;
                lineStyle = check_Jurbol_TDI_DSMA.LineStyle;
                check_Jurbol_TDI_DSMA.Period = period;
                period = check_Jurbol_TDI_DSMA.Period;
                check_Jurbol_TDI_DSMA.PriceLine = priceLine;
                priceLine = check_Jurbol_TDI_DSMA.PriceLine;
                check_Jurbol_TDI_DSMA.RayLength = rayLength;
                rayLength = check_Jurbol_TDI_DSMA.RayLength;
                check_Jurbol_TDI_DSMA.Up = up;
                up = check_Jurbol_TDI_DSMA.Up;

                if (cache_Jurbol_TDI_DSMA != null)
                    for (int idx = 0; idx < cache_Jurbol_TDI_DSMA.Length; idx++)
                        if (cache_Jurbol_TDI_DSMA[idx].Colors == colors && cache_Jurbol_TDI_DSMA[idx].Displace == displace && cache_Jurbol_TDI_DSMA[idx].Dn == dn && cache_Jurbol_TDI_DSMA[idx].LineColorBear == lineColorBear && cache_Jurbol_TDI_DSMA[idx].LineColorBull == lineColorBull && cache_Jurbol_TDI_DSMA[idx].LineColorFlat == lineColorFlat && cache_Jurbol_TDI_DSMA[idx].LineStyle == lineStyle && cache_Jurbol_TDI_DSMA[idx].Period == period && cache_Jurbol_TDI_DSMA[idx].PriceLine == priceLine && cache_Jurbol_TDI_DSMA[idx].RayLength == rayLength && cache_Jurbol_TDI_DSMA[idx].Up == up && cache_Jurbol_TDI_DSMA[idx].EqualsInput(input))
                            return cache_Jurbol_TDI_DSMA[idx];

                _Jurbol_TDI_DSMA indicator = new _Jurbol_TDI_DSMA();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.Colors = colors;
                indicator.Displace = displace;
                indicator.Dn = dn;
                indicator.LineColorBear = lineColorBear;
                indicator.LineColorBull = lineColorBull;
                indicator.LineColorFlat = lineColorFlat;
                indicator.LineStyle = lineStyle;
                indicator.Period = period;
                indicator.PriceLine = priceLine;
                indicator.RayLength = rayLength;
                indicator.Up = up;
                Indicators.Add(indicator);
                indicator.SetUp();

                _Jurbol_TDI_DSMA[] tmp = new _Jurbol_TDI_DSMA[cache_Jurbol_TDI_DSMA == null ? 1 : cache_Jurbol_TDI_DSMA.Length + 1];
                if (cache_Jurbol_TDI_DSMA != null)
                    cache_Jurbol_TDI_DSMA.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cache_Jurbol_TDI_DSMA = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Displaced SMA indicators with colorcoded close above/below the DSMA. Selectable priceline also included.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator._Jurbol_TDI_DSMA _Jurbol_TDI_DSMA(bool colors, int displace, Color dn, Color lineColorBear, Color lineColorBull, Color lineColorFlat, DashStyle lineStyle, int period, bool priceLine, int rayLength, Color up)
        {
            return _indicator._Jurbol_TDI_DSMA(Input, colors, displace, dn, lineColorBear, lineColorBull, lineColorFlat, lineStyle, period, priceLine, rayLength, up);
        }

        /// <summary>
        /// Displaced SMA indicators with colorcoded close above/below the DSMA. Selectable priceline also included.
        /// </summary>
        /// <returns></returns>
        public Indicator._Jurbol_TDI_DSMA _Jurbol_TDI_DSMA(Data.IDataSeries input, bool colors, int displace, Color dn, Color lineColorBear, Color lineColorBull, Color lineColorFlat, DashStyle lineStyle, int period, bool priceLine, int rayLength, Color up)
        {
            return _indicator._Jurbol_TDI_DSMA(input, colors, displace, dn, lineColorBear, lineColorBull, lineColorFlat, lineStyle, period, priceLine, rayLength, up);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Displaced SMA indicators with colorcoded close above/below the DSMA. Selectable priceline also included.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator._Jurbol_TDI_DSMA _Jurbol_TDI_DSMA(bool colors, int displace, Color dn, Color lineColorBear, Color lineColorBull, Color lineColorFlat, DashStyle lineStyle, int period, bool priceLine, int rayLength, Color up)
        {
            return _indicator._Jurbol_TDI_DSMA(Input, colors, displace, dn, lineColorBear, lineColorBull, lineColorFlat, lineStyle, period, priceLine, rayLength, up);
        }

        /// <summary>
        /// Displaced SMA indicators with colorcoded close above/below the DSMA. Selectable priceline also included.
        /// </summary>
        /// <returns></returns>
        public Indicator._Jurbol_TDI_DSMA _Jurbol_TDI_DSMA(Data.IDataSeries input, bool colors, int displace, Color dn, Color lineColorBear, Color lineColorBull, Color lineColorFlat, DashStyle lineStyle, int period, bool priceLine, int rayLength, Color up)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator._Jurbol_TDI_DSMA(input, colors, displace, dn, lineColorBear, lineColorBull, lineColorFlat, lineStyle, period, priceLine, rayLength, up);
        }
    }
}
#endregion
