#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

//*********************************************************************
// Programmed by Finn Bjurvoll Hansen - www.jurbol.com
// Email: jurboltrading@gmail.com
//*********************************************************************

namespace NinjaTrader.Indicator
{
   
    [Description("TDI Indicator. ")]
    public class _Jurbol_TDI_V1 : Indicator
    {
        #region Variables
        //***************************************
		// BOOLS
		//***************************************
		private bool		showCrossArrows		     = true;
		private bool		showBounceArrows		 = false;
		private bool		showCrossLines			 = true;
		private bool		showBounceLines			 = false;
		//***************************************
		// ARROWS SETTINGS
		//***************************************
        private int 		arrowSize 				 = 12;
		private int 		offset 				 	 = 30;
		//***************************************
		// COLORS
		//***************************************
		private Color		crossUp		   	         = Color.LimeGreen;
		private Color		crossDn		   	         = Color.Red;
		private Color		bounceUp		   	     = Color.Blue;
		private Color		bounceDn		   	     = Color.Magenta;
		//***************************************
		// AUDIO ALERTS
		//***************************************
       
        #endregion

        
        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.Lime), PlotStyle.Line, "01. TDI"));
            Add(new Plot(Color.FromKnownColor(KnownColor.Red), PlotStyle.Line, "02. TDMA"));
			Add(new Plot(Color.FromKnownColor(KnownColor.Red), PlotStyle.Line, "03. UPPER LINE"));
			Add(new Plot(Color.FromKnownColor(KnownColor.Black), PlotStyle.Line, "04. MIDDLE LINE"));
			Add(new Plot(Color.FromKnownColor(KnownColor.Green), PlotStyle.Line, "05. LOWER LINE"));
			//Add(new Plot(Color.FromKnownColor(KnownColor.Green), PlotStyle.Bar, "06. DIFF HISTOGRAM"));
			
			Plots[0].Pen.Width		= 2;
			Plots[1].Pen.Width		= 2;
			Plots[2].Pen.Width		= 1;
			Plots[2].Pen.DashStyle	= DashStyle.Dash;
			Plots[3].Pen.Width		= 1;
			Plots[3].Pen.DashStyle	= DashStyle.Dash;
			Plots[4].Pen.Width		= 1;
			Plots[4].Pen.DashStyle	= DashStyle.Dash;
			//Plots[5].Pen.Width		= 2;
			
            DisplayInDataBox 	= false; 
			PaintPriceMarkers 	= true; 
            CalculateOnBarClose	= false;
            Overlay				= false;
            PriceTypeSupported	= true;
			DrawOnPricePanel	= false;
			HorizontalGridLines = false;
			VerticalGridLines 	= false;
        }

       
        protected override void OnBarUpdate()
         {if 
		(CurrentBar < 2) 
				return;
          
            TDI.Set(SMA(RSI(13, 1), 2)[0]);
            TDMA.Set(SMA(SMA(RSI(13, 1), 2), 7)[0]);
			Upper.Set(68);
			Middle.Set(50);
			Lower.Set(32);
			//Diff.Set(SMA(RSI(13, 1), 2)[0] - SMA(SMA(RSI(13, 1), 2), 7)[0]);
		
		//**********************************************************************
		// ALERTS CROSSES PLOTTING ON INDICATOR
		//**********************************************************************
		DrawOnPricePanel	= false;
		if(showCrossLines)
		if(TDI[1] < TDMA[1] && TDI[0] > TDMA[0])
		{
		DrawVerticalLine("CrossUp"+CurrentBar, 0, crossUp);
		}
		
		else if(TDI[1] > TDMA[1] && TDI[0] < TDMA[0])
		{
		DrawVerticalLine("CrossDn"+CurrentBar, 0, crossDn);
		}
		else
		{
		RemoveDrawObject("CrossUp"+CurrentBar);
		RemoveDrawObject("CrossDn"+CurrentBar);
		}
		
		//**********************************************************************
		// ALERTS BOUNCES PLOTTING ON INDICATOR
		//**********************************************************************
		if(showBounceLines)
		if(TDI[1] < TDI[2] && TDI[0] > TDI[1] && TDI[1] > TDMA[1])
		{
		DrawVerticalLine("BounceUp"+CurrentBar, 0, bounceUp);
		}
		else if(TDI[1] > TDI[2] && TDI[0] < TDI[1] && TDI[1] < TDMA[1])
		{
		DrawVerticalLine("BounceDn"+CurrentBar, 0, bounceDn);
		}
		else
		{
		RemoveDrawObject("BounceUp"+CurrentBar);
		RemoveDrawObject("BounceDn"+CurrentBar);
		}
		
		//**********************************************************************
		// ALERTS CROSSES PLOTTING ON PRICECHART AS ARROWS
		//**********************************************************************
		DrawOnPricePanel	= true;
		if(showCrossArrows)
		if(TDI[1] < TDMA[1] && TDI[0] > TDMA[0])
		{
		DrawText("UpArrow"+CurrentBar, false, "é", 0, Low[0] -offset *TickSize,0, crossUp, new Font("Wingdings", arrowSize, FontStyle.Regular), StringAlignment.Center, Color.Transparent, Color.Transparent, 0);
		}
		
		else if(TDI[1] > TDMA[1] && TDI[0] < TDMA[0])
		{
		DrawText("DnArrow"+CurrentBar, false, "ê", 0, High[0] +offset *TickSize,0, crossDn, new Font("Wingdings", arrowSize, FontStyle.Regular), StringAlignment.Center, Color.Transparent, Color.Transparent, 0);

		}
		else
		{
		RemoveDrawObject("UpArrow"+CurrentBar);
		RemoveDrawObject("DnArrow"+CurrentBar);
		}
		//**********************************************************************
		// ALERTS BOUNCES PLOTTING ON PRICECHART AS ARROWS
		//**********************************************************************
		if(showBounceArrows)
        if(TDI[1] < TDI[2] && TDI[0] > TDI[1] && TDI[1] > TDMA[1])
		{
		DrawText("BounceUpArrow"+CurrentBar, false, "é", 0, Low[0] -offset *TickSize,0, bounceUp, new Font("Wingdings", arrowSize, FontStyle.Regular), StringAlignment.Center, Color.Transparent, Color.Transparent, 0);
		}
		else if(TDI[1] > TDI[2] && TDI[0] < TDI[1] && TDI[1] < TDMA[1])
		{
		DrawText("BounceDnArrow"+CurrentBar, false, "ê", 0, High[0] +offset *TickSize,0, bounceDn, new Font("Wingdings", arrowSize, FontStyle.Regular), StringAlignment.Center, Color.Transparent, Color.Transparent, 0);
		}
		else
		{
		RemoveDrawObject("BounceUpArrow"+CurrentBar);
		RemoveDrawObject("BounceDnArrow"+CurrentBar);
		}
		
		}
		public override string ToString()
		{
			Name= "TDI © Jurbol.com";
			return Name;
        }

        #region Properties
        [Browsable(false)]
        [XmlIgnore()]
        public DataSeries TDI
        {
            get { return Values[0]; }
        }

        [Browsable(false)]
        [XmlIgnore()]
        public DataSeries TDMA
        {
            get { return Values[1]; }
        }
		
		[Browsable(false)]
        [XmlIgnore()]
        public DataSeries Upper
        {
            get { return Values[2]; }
        }
		
		[Browsable(false)]
        [XmlIgnore()]
        public DataSeries Middle
        {
            get { return Values[3]; }
        }
		
		[Browsable(false)]
        [XmlIgnore()]
        public DataSeries Lower
        {
            get { return Values[4]; }
        }
		
//		[Browsable(false)]
//        [XmlIgnore()]
//        public DataSeries Diff
//        {
//            get { return Values[5]; }
//        }
		
		//**************************************************************
		// SETTINGS
		//*************************************************************

        [Description("Display vertical lines that indicates crosses")]
		[GridCategory("01. Visual Alerts")]
		[Gui.Design.DisplayNameAttribute("01. Show CrossLines")]
		public bool ShowCrossLines
		{
			get { return showCrossLines; }
			set { showCrossLines = value; }
		}
		
		[Description("Display vertical lines that indicates bounces")]
		[GridCategory("01. Visual Alerts")]
		[Gui.Design.DisplayNameAttribute("02. Show BounceLines")]
		public bool ShowBounceLines
		{
			get { return showBounceLines; }
			set { showBounceLines = value; }
		}
		
		[Description("Display arrows on price chart that indicates crosses")]
		[GridCategory("01. Visual Alerts")]
		[Gui.Design.DisplayNameAttribute("03. Show Cross Arrows")]
		public bool ShowCrossArrows
		{
			get { return showCrossArrows; }
			set { showCrossArrows = value; }
		}
		
		[Description("Display arrows on price chart that indicates bounces")]
		[GridCategory("01. Visual Alerts")]
		[Gui.Design.DisplayNameAttribute("04. Show Bounce Arrows")]
		public bool ShowBounceArrows
		{
			get { return showBounceArrows; }
			set { showBounceArrows = value; }
		}
		
		[Description("Set the arrows size (pix)")]
		[GridCategory("01. Visual Alerts")]
		[Gui.Design.DisplayNameAttribute("05. Arrow Size")]
		public int ArrowSize
		{
			get{return arrowSize;}
			set{arrowSize = Math.Max(0,value);}
		}
		
		[Description("Set the arrows offset from the bars (pips/ticks)")]
		[GridCategory("01. Visual Alerts")]
		[Gui.Design.DisplayNameAttribute("06. Arrow Offset From Bar")]
		public int Offset
		{
			get{return offset;}
			set{offset = Math.Max(0,value);}
		}
		
		//***************************************************
		// COLORS
		//***************************************************
		
		[XmlIgnore()]
        [Description("Color for TDI Crossing up")]
        [GridCategory("02. Colors")]
		[Gui.Design.DisplayNameAttribute("01. Crossing Up")]
        public Color CrossUp
        {
            get { return crossUp; }
            set { crossUp = value; }
        }
		[Browsable(false)]
		public string CrossUpSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(CrossUp); }
			set { CrossUp = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		[XmlIgnore()]
        [Description("Color for TDI Crossing down")]
        [GridCategory("02. Colors")]
		[Gui.Design.DisplayNameAttribute("02. Crossing Dn")]
        public Color CrossDn
        {
            get { return crossDn; }
            set { crossDn = value; }
        }
		[Browsable(false)]
		public string CrossDnSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(CrossDn); }
			set { CrossDn = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		[XmlIgnore()]
        [Description("Color for TDI Bouncing up")]
        [GridCategory("02. Colors")]
		[Gui.Design.DisplayNameAttribute("03. Bounce Up")]
        public Color BounceUp
        {
            get { return bounceUp; }
            set { bounceUp = value; }
        }
		[Browsable(false)]
		public string BounceUpSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(BounceUp); }
			set { BounceUp = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		[XmlIgnore()]
        [Description("Color for TDI Bouncing down")]
        [GridCategory("02. Colors")]
		[Gui.Design.DisplayNameAttribute("04. Bounce Dn")]
        public Color BounceDn
        {
            get { return bounceDn; }
            set { bounceDn = value; }
        }
		[Browsable(false)]
		public string BounceDnSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(BounceDn); }
			set { BounceDn = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private _Jurbol_TDI_V1[] cache_Jurbol_TDI_V1 = null;

        private static _Jurbol_TDI_V1 check_Jurbol_TDI_V1 = new _Jurbol_TDI_V1();

        /// <summary>
        /// TDI Indicator. 
        /// </summary>
        /// <returns></returns>
        public _Jurbol_TDI_V1 _Jurbol_TDI_V1(int arrowSize, Color bounceDn, Color bounceUp, Color crossDn, Color crossUp, int offset, bool showBounceArrows, bool showBounceLines, bool showCrossArrows, bool showCrossLines)
        {
            return _Jurbol_TDI_V1(Input, arrowSize, bounceDn, bounceUp, crossDn, crossUp, offset, showBounceArrows, showBounceLines, showCrossArrows, showCrossLines);
        }

        /// <summary>
        /// TDI Indicator. 
        /// </summary>
        /// <returns></returns>
        public _Jurbol_TDI_V1 _Jurbol_TDI_V1(Data.IDataSeries input, int arrowSize, Color bounceDn, Color bounceUp, Color crossDn, Color crossUp, int offset, bool showBounceArrows, bool showBounceLines, bool showCrossArrows, bool showCrossLines)
        {
            if (cache_Jurbol_TDI_V1 != null)
                for (int idx = 0; idx < cache_Jurbol_TDI_V1.Length; idx++)
                    if (cache_Jurbol_TDI_V1[idx].ArrowSize == arrowSize && cache_Jurbol_TDI_V1[idx].BounceDn == bounceDn && cache_Jurbol_TDI_V1[idx].BounceUp == bounceUp && cache_Jurbol_TDI_V1[idx].CrossDn == crossDn && cache_Jurbol_TDI_V1[idx].CrossUp == crossUp && cache_Jurbol_TDI_V1[idx].Offset == offset && cache_Jurbol_TDI_V1[idx].ShowBounceArrows == showBounceArrows && cache_Jurbol_TDI_V1[idx].ShowBounceLines == showBounceLines && cache_Jurbol_TDI_V1[idx].ShowCrossArrows == showCrossArrows && cache_Jurbol_TDI_V1[idx].ShowCrossLines == showCrossLines && cache_Jurbol_TDI_V1[idx].EqualsInput(input))
                        return cache_Jurbol_TDI_V1[idx];

            lock (check_Jurbol_TDI_V1)
            {
                check_Jurbol_TDI_V1.ArrowSize = arrowSize;
                arrowSize = check_Jurbol_TDI_V1.ArrowSize;
                check_Jurbol_TDI_V1.BounceDn = bounceDn;
                bounceDn = check_Jurbol_TDI_V1.BounceDn;
                check_Jurbol_TDI_V1.BounceUp = bounceUp;
                bounceUp = check_Jurbol_TDI_V1.BounceUp;
                check_Jurbol_TDI_V1.CrossDn = crossDn;
                crossDn = check_Jurbol_TDI_V1.CrossDn;
                check_Jurbol_TDI_V1.CrossUp = crossUp;
                crossUp = check_Jurbol_TDI_V1.CrossUp;
                check_Jurbol_TDI_V1.Offset = offset;
                offset = check_Jurbol_TDI_V1.Offset;
                check_Jurbol_TDI_V1.ShowBounceArrows = showBounceArrows;
                showBounceArrows = check_Jurbol_TDI_V1.ShowBounceArrows;
                check_Jurbol_TDI_V1.ShowBounceLines = showBounceLines;
                showBounceLines = check_Jurbol_TDI_V1.ShowBounceLines;
                check_Jurbol_TDI_V1.ShowCrossArrows = showCrossArrows;
                showCrossArrows = check_Jurbol_TDI_V1.ShowCrossArrows;
                check_Jurbol_TDI_V1.ShowCrossLines = showCrossLines;
                showCrossLines = check_Jurbol_TDI_V1.ShowCrossLines;

                if (cache_Jurbol_TDI_V1 != null)
                    for (int idx = 0; idx < cache_Jurbol_TDI_V1.Length; idx++)
                        if (cache_Jurbol_TDI_V1[idx].ArrowSize == arrowSize && cache_Jurbol_TDI_V1[idx].BounceDn == bounceDn && cache_Jurbol_TDI_V1[idx].BounceUp == bounceUp && cache_Jurbol_TDI_V1[idx].CrossDn == crossDn && cache_Jurbol_TDI_V1[idx].CrossUp == crossUp && cache_Jurbol_TDI_V1[idx].Offset == offset && cache_Jurbol_TDI_V1[idx].ShowBounceArrows == showBounceArrows && cache_Jurbol_TDI_V1[idx].ShowBounceLines == showBounceLines && cache_Jurbol_TDI_V1[idx].ShowCrossArrows == showCrossArrows && cache_Jurbol_TDI_V1[idx].ShowCrossLines == showCrossLines && cache_Jurbol_TDI_V1[idx].EqualsInput(input))
                            return cache_Jurbol_TDI_V1[idx];

                _Jurbol_TDI_V1 indicator = new _Jurbol_TDI_V1();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.ArrowSize = arrowSize;
                indicator.BounceDn = bounceDn;
                indicator.BounceUp = bounceUp;
                indicator.CrossDn = crossDn;
                indicator.CrossUp = crossUp;
                indicator.Offset = offset;
                indicator.ShowBounceArrows = showBounceArrows;
                indicator.ShowBounceLines = showBounceLines;
                indicator.ShowCrossArrows = showCrossArrows;
                indicator.ShowCrossLines = showCrossLines;
                Indicators.Add(indicator);
                indicator.SetUp();

                _Jurbol_TDI_V1[] tmp = new _Jurbol_TDI_V1[cache_Jurbol_TDI_V1 == null ? 1 : cache_Jurbol_TDI_V1.Length + 1];
                if (cache_Jurbol_TDI_V1 != null)
                    cache_Jurbol_TDI_V1.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cache_Jurbol_TDI_V1 = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// TDI Indicator. 
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator._Jurbol_TDI_V1 _Jurbol_TDI_V1(int arrowSize, Color bounceDn, Color bounceUp, Color crossDn, Color crossUp, int offset, bool showBounceArrows, bool showBounceLines, bool showCrossArrows, bool showCrossLines)
        {
            return _indicator._Jurbol_TDI_V1(Input, arrowSize, bounceDn, bounceUp, crossDn, crossUp, offset, showBounceArrows, showBounceLines, showCrossArrows, showCrossLines);
        }

        /// <summary>
        /// TDI Indicator. 
        /// </summary>
        /// <returns></returns>
        public Indicator._Jurbol_TDI_V1 _Jurbol_TDI_V1(Data.IDataSeries input, int arrowSize, Color bounceDn, Color bounceUp, Color crossDn, Color crossUp, int offset, bool showBounceArrows, bool showBounceLines, bool showCrossArrows, bool showCrossLines)
        {
            return _indicator._Jurbol_TDI_V1(input, arrowSize, bounceDn, bounceUp, crossDn, crossUp, offset, showBounceArrows, showBounceLines, showCrossArrows, showCrossLines);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// TDI Indicator. 
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator._Jurbol_TDI_V1 _Jurbol_TDI_V1(int arrowSize, Color bounceDn, Color bounceUp, Color crossDn, Color crossUp, int offset, bool showBounceArrows, bool showBounceLines, bool showCrossArrows, bool showCrossLines)
        {
            return _indicator._Jurbol_TDI_V1(Input, arrowSize, bounceDn, bounceUp, crossDn, crossUp, offset, showBounceArrows, showBounceLines, showCrossArrows, showCrossLines);
        }

        /// <summary>
        /// TDI Indicator. 
        /// </summary>
        /// <returns></returns>
        public Indicator._Jurbol_TDI_V1 _Jurbol_TDI_V1(Data.IDataSeries input, int arrowSize, Color bounceDn, Color bounceUp, Color crossDn, Color crossUp, int offset, bool showBounceArrows, bool showBounceLines, bool showCrossArrows, bool showCrossLines)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator._Jurbol_TDI_V1(input, arrowSize, bounceDn, bounceUp, crossDn, crossUp, offset, showBounceArrows, showBounceLines, showCrossArrows, showCrossLines);
        }
    }
}
#endregion
