#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

//*********************************************************************
// Programmed by Finn Bjurvoll Hansen - www.jurbol.com
// Email: jurboltrading@gmail.com
//*********************************************************************
namespace NinjaTrader.Indicator
{
    
    [Description("Histogram view of the _Jurbol_TDI_V1")]
    public class _Jurbol_TDI_V1_Hist : Indicator
    {
        #region Variables
		private DataSeries  TDI;
		private DataSeries  TDMA;
		
		private Color		up	   	         = Color.Lime;
		private Color		dn		   	     = Color.Red;
        #endregion

       
        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.Orange), PlotStyle.Bar, "TDI Histogram"));
			Plots[0].Pen.Width = 10;
			
			DisplayInDataBox 	= false; 
			PaintPriceMarkers 	= true; 
            CalculateOnBarClose	= false;
            Overlay				= false;
            PriceTypeSupported	= true;
			DrawOnPricePanel	= false;
            Overlay				= false;
			
			TDI	  	  = new DataSeries(this);
			TDMA	  = new DataSeries(this);
        }

        protected override void OnBarUpdate()
        {
           
			
			TDI.Set( SMA(RSI(13, 1), 2)[0]);
			TDMA.Set(SMA(SMA(RSI(13, 1), 2), 7)[0]);
            Plot0.Set(Close[0]);
			
			if(TDI[0] > TDMA[0] ){PlotColors[0][0] = up;}
			else if(TDI[0] < TDMA[0] ){PlotColors[0][0] = dn;}
			else {PlotColors[0][0] = Color.Orange;}
				
        }
		public override string ToString()
		{
			Name= "TDI HIST © Jurbol.com";
			return Name;
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Plot0
        {
            get { return Values[0]; }
        }

       
		
		[XmlIgnore()]
        [Description("Color for close above DSMA")]
        [GridCategory("01. Colors")]
		[Gui.Design.DisplayNameAttribute("01. Up")]
        public Color Up
        {
            get { return up; }
            set { up = value; }
        }
		[Browsable(false)]
		public string UpSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(Up); }
			set { Up = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		[XmlIgnore()]
        [Description("Color for close below DSMA")]
        [GridCategory("01. Colors")]
		[Gui.Design.DisplayNameAttribute("02. Down")]
        public Color Dn
        {
            get { return dn; }
            set { dn = value; }
        }
		[Browsable(false)]
		public string DnSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(Dn); }
			set { Dn = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private _Jurbol_TDI_V1_Hist[] cache_Jurbol_TDI_V1_Hist = null;

        private static _Jurbol_TDI_V1_Hist check_Jurbol_TDI_V1_Hist = new _Jurbol_TDI_V1_Hist();

        /// <summary>
        /// Histogram view of the _Jurbol_TDI_V1
        /// </summary>
        /// <returns></returns>
        public _Jurbol_TDI_V1_Hist _Jurbol_TDI_V1_Hist(Color dn, Color up)
        {
            return _Jurbol_TDI_V1_Hist(Input, dn, up);
        }

        /// <summary>
        /// Histogram view of the _Jurbol_TDI_V1
        /// </summary>
        /// <returns></returns>
        public _Jurbol_TDI_V1_Hist _Jurbol_TDI_V1_Hist(Data.IDataSeries input, Color dn, Color up)
        {
            if (cache_Jurbol_TDI_V1_Hist != null)
                for (int idx = 0; idx < cache_Jurbol_TDI_V1_Hist.Length; idx++)
                    if (cache_Jurbol_TDI_V1_Hist[idx].Dn == dn && cache_Jurbol_TDI_V1_Hist[idx].Up == up && cache_Jurbol_TDI_V1_Hist[idx].EqualsInput(input))
                        return cache_Jurbol_TDI_V1_Hist[idx];

            lock (check_Jurbol_TDI_V1_Hist)
            {
                check_Jurbol_TDI_V1_Hist.Dn = dn;
                dn = check_Jurbol_TDI_V1_Hist.Dn;
                check_Jurbol_TDI_V1_Hist.Up = up;
                up = check_Jurbol_TDI_V1_Hist.Up;

                if (cache_Jurbol_TDI_V1_Hist != null)
                    for (int idx = 0; idx < cache_Jurbol_TDI_V1_Hist.Length; idx++)
                        if (cache_Jurbol_TDI_V1_Hist[idx].Dn == dn && cache_Jurbol_TDI_V1_Hist[idx].Up == up && cache_Jurbol_TDI_V1_Hist[idx].EqualsInput(input))
                            return cache_Jurbol_TDI_V1_Hist[idx];

                _Jurbol_TDI_V1_Hist indicator = new _Jurbol_TDI_V1_Hist();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.Dn = dn;
                indicator.Up = up;
                Indicators.Add(indicator);
                indicator.SetUp();

                _Jurbol_TDI_V1_Hist[] tmp = new _Jurbol_TDI_V1_Hist[cache_Jurbol_TDI_V1_Hist == null ? 1 : cache_Jurbol_TDI_V1_Hist.Length + 1];
                if (cache_Jurbol_TDI_V1_Hist != null)
                    cache_Jurbol_TDI_V1_Hist.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cache_Jurbol_TDI_V1_Hist = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Histogram view of the _Jurbol_TDI_V1
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator._Jurbol_TDI_V1_Hist _Jurbol_TDI_V1_Hist(Color dn, Color up)
        {
            return _indicator._Jurbol_TDI_V1_Hist(Input, dn, up);
        }

        /// <summary>
        /// Histogram view of the _Jurbol_TDI_V1
        /// </summary>
        /// <returns></returns>
        public Indicator._Jurbol_TDI_V1_Hist _Jurbol_TDI_V1_Hist(Data.IDataSeries input, Color dn, Color up)
        {
            return _indicator._Jurbol_TDI_V1_Hist(input, dn, up);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Histogram view of the _Jurbol_TDI_V1
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator._Jurbol_TDI_V1_Hist _Jurbol_TDI_V1_Hist(Color dn, Color up)
        {
            return _indicator._Jurbol_TDI_V1_Hist(Input, dn, up);
        }

        /// <summary>
        /// Histogram view of the _Jurbol_TDI_V1
        /// </summary>
        /// <returns></returns>
        public Indicator._Jurbol_TDI_V1_Hist _Jurbol_TDI_V1_Hist(Data.IDataSeries input, Color dn, Color up)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator._Jurbol_TDI_V1_Hist(input, dn, up);
        }
    }
}
#endregion
